@@ -3,6 +3,7 @@
 
 #include <SRMTypes.h>
 #include <GLES2/gl2.h>
+#include <EGL/egl.h>
 #include <gbm.h>
 
 #ifdef __cplusplus
@@ -74,7 +75,12 @@ enum SRM_BUFFER_SRC
     /**
      * The buffer source is a bo from the Generic Buffer Manager (GBM) library.
      */
-    SRM_BUFFER_SRC_GBM
+    SRM_BUFFER_SRC_GBM,
+
+    /**
+     * The buffer source is an already existing OpenGL texture.
+     */
+    SRM_BUFFER_SRC_GL
 };
 
 /**
@@ -133,8 +139,8 @@ typedef struct SRMBufferDMADataStruct
  *
  * This function creates an @ref SRMBuffer object using the provided 
  * [Generic Buffer Management (GBM)](https://en.wikipedia.org/wiki/Mesa_(computer_graphics)#Generic_Buffer_Management) buffer object.
- * 
- * @warning The `gbm_bo` must remain valid during the buffer lifetime. It is not destroyed when 
+ *
+ * @warning The `struct gbm_bo` must remain valid during the buffer lifetime. It is not destroyed when
  *          the buffer is destroyed and must be destroyed manually.
  *
  * @note Depending on the backing storage of the bo, the buffer may not be shareable across all devices.
@@ -200,6 +206,25 @@ SRMBuffer *srmBufferCreateFromCPU(SRMCore *core, SRMDevice *allocator,
  */
 SRMBuffer *srmBufferCreateFromWaylandDRM(SRMCore *core, void *wlBuffer);
 
+/**
+ * @brief Creates a wrapper for an already created OpenGL texture.
+ *
+ * This function allows you to use an existing OpenGL texture within the SRM framework.
+ *
+ * @note The texture can only be used by the provided @ref SRMDevice.
+ *
+ * @param device The @ref SRMDevice whose `EGLDisplay` was used to create the texture.
+ * @param id The OpenGL texture ID of the existing texture.
+ * @param target The target for the texture (e.g., `GL_TEXTURE_2D` or `GL_TEXTURE_EXTERNAL_OES`).
+ * @param format The DRM format of the texture.
+ * @param width The width of the texture in pixels.
+ * @param height The height of the texture in pixels.
+ * @param transferOwnership If non-zero, the texture will be destroyed when @ref srmBufferDestroy() is called.
+ * @return A pointer to the created @ref SRMBuffer, or `NULL` on failure.
+ */
+SRMBuffer *srmBufferCreateGLTextureWrapper(SRMDevice *device, GLuint id, GLenum target, SRM_BUFFER_FORMAT format,
+                                           UInt32 width, UInt32 height, UInt8 transferOwnership);
+
 /**
  * @brief Destroys an SRMBuffer.
  *
@@ -225,7 +250,7 @@ void srmBufferDestroy(SRMBuffer *buffer);
 GLuint srmBufferGetTextureID(SRMDevice *device, SRMBuffer *buffer);
 
 /**
- * @brief Get the OpenGL texture target associated with an @ref SRMBuffer.
+ * @brief Gets the OpenGL texture target associated with an @ref SRMBuffer.
  *
  * This function retrieves the OpenGL texture target associated with the provided @ref SRMBuffer.
  * 
@@ -250,18 +275,32 @@ GLuint srmBufferGetTextureID(SRMDevice *device, SRMBuffer *buffer);
  */
 GLenum srmBufferGetTextureTarget(SRMBuffer *buffer);
 
+/**
+ * @brief Gets an `EGLImage` associated with a specific device.
+ *
+ * The returned image is owned by the SRMBuffer and should not be destroyed manually.
+ *
+ * @param device The device whose `EGLDisplay` is used to create the image.
+ * @param buffer Pointer to the @ref SRMBuffer.
+ *
+ * @return An `EGLImage` associated with the SRMDevice or `EGL_NO_IMAGE` on failure.
+ */
+EGLImage srmBufferGetEGLImage(SRMDevice *device, SRMBuffer *buffer);
+
 /**
  * @brief Writes pixel data to an @ref SRMBuffer.
  *
  * This function writes pixel data to the specified region of an @ref SRMBuffer.
+ *
+ * @note The source and destination buffer pixel format must match.
  * 
  * @param buffer Pointer to the @ref SRMBuffer to write data to.
- * @param stride Stride (row pitch) of the source pixel data.
+ * @param stride Stride of the source pixel data.
  * @param dstX X-coordinate of the top-left corner of the destination region.
  * @param dstY Y-coordinate of the top-left corner of the destination region.
  * @param dstWidth Width of the destination region.
  * @param dstHeight Height of the destination region.
- * @param pixels Pointer to the source pixel data. Must point to the top-left of the source buffer.
+ * @param pixels Pointer within the source buffer to copy from (not affected by dstX and dstY).
  * @return 1 on success, or 0 on failure.
  */
 UInt8 srmBufferWrite(SRMBuffer *buffer,
@@ -276,7 +315,10 @@ UInt8 srmBufferWrite(SRMBuffer *buffer,
  * @brief Reads pixel data from an @ref SRMBuffer.
  *
  * This function reads pixel data from the specified region of an @ref SRMBuffer and copies it to the destination buffer.
- * 
+ *
+ * @note On failure, the texture should still be readable by attaching the texture ID returned by @ref srmBufferGetTextureID()
+ *       to a GL framebuffer and using `glReadPixels()`.
+ *
  * @param buffer Pointer to the @ref SRMBuffer to read data from.
  * @param srcX X-coordinate of the top-left corner of the source region.
  * @param srcY Y-coordinate of the top-left corner of the source region.
