@@ -295,9 +295,9 @@ UInt8 srmConnectorInitialize(SRMConnector *connector, SRMConnectorInterface *int
     if (!srmConnectorGetBestConfiguration(connector, &bestEncoder, &bestCrtc, &bestPrimaryPlane, &bestCursorPlane))
     {
         // Fails to get a valid encoder, crtc or primary plane
-        SRMWarning("Could not get a Encoder, Crtc and Primary Plane trio for device %s connector %d.",
-                   connector->device->name,
-                   connector->id);
+        SRMWarning("[%s] [%s] Could not get a Encoder, Crtc and Primary Plane trio.",
+                   connector->device->shortName,
+                   connector->name);
         return 0;
     }
 
@@ -320,7 +320,7 @@ UInt8 srmConnectorInitialize(SRMConnector *connector, SRMConnectorInterface *int
     else
         connector->currentCursorPlane = NULL;
 
-
+    connector->fenceFD = -1;
     connector->interfaceData = userData;
     connector->interface = interface;
 
@@ -331,7 +331,7 @@ UInt8 srmConnectorInitialize(SRMConnector *connector, SRMConnectorInterface *int
 
     if (pthread_create(&connector->renderThread, NULL, srmConnectorRenderThread, connector))
     {
-        SRMError("Could not start render thread for device %s connector %d.", connector->device->name, connector->id);
+        SRMError("[%s] [%s] Could not start rendering thread.", connector->device->shortName, connector->name);
         goto fail;
     }
 
@@ -343,12 +343,7 @@ UInt8 srmConnectorInitialize(SRMConnector *connector, SRMConnectorInterface *int
 
     connector->state = SRM_CONNECTOR_STATE_INITIALIZED;
 
-    SRMDebug("[%s] Connector (%d) %s, %s, %s initialized.",
-             connector->device->name,
-             connector->id,
-             connector->name,
-             connector->model,
-             connector->manufacturer);
+    SRMDebug("[%s] [%s] Initialized.", connector->device->shortName, connector->name);
 
     return 1;
 
@@ -403,12 +398,7 @@ void srmConnectorUninitialize(SRMConnector *connector)
 
     srmConnectorRenderThreadCleanUp(connector);
 
-    SRMDebug("[%s] Connector (%d) %s, %s, %s uninitialized.",
-             connector->device->name,
-             connector->id,
-             connector->name,
-             connector->model,
-             connector->manufacturer);
+    SRMDebug("[%s] [%s] Uninitialized.", connector->device->shortName, connector->name);
 }
 
 UInt32 srmConnectorGetCurrentBufferIndex(SRMConnector *connector)
@@ -742,8 +732,8 @@ UInt8 srmConnectorSetCustomScanoutBuffer(SRMConnector *connector, SRMBuffer *buf
 
     if (buffer == connector->userScanoutBufferRef[0])
     {
-        SRMDebug("[%s][%s] Custom scanout buffer succesfully set.",
-                 connector->device->name,
+        SRMDebug("[%s] [%s] Custom scanout buffer succesfully set.",
+                 connector->device->shortName,
                  connector->name);
         return 1;
     }
@@ -752,16 +742,16 @@ UInt8 srmConnectorSetCustomScanoutBuffer(SRMConnector *connector, SRMBuffer *buf
 
     if (!buffer)
     {
-        SRMDebug("[%s][%s] Custom scanout buffer succesfully unset.",
-                 connector->device->name,
+        SRMDebug("[%s] [%s] Custom scanout buffer succesfully unset.",
+                 connector->device->shortName,
                  connector->name);
         return 0;
     }
 
     if (connector->device != buffer->allocator)
     {
-        SRMError("[%s][%s] Failed to set custom scanout buffer. The buffer allocator must match the connector's device.",
-                 connector->device->name,
+        SRMError("[%s] [%s] Failed to set custom scanout buffer. The buffer allocator must match the connector's device.",
+                 connector->device->shortName,
                  connector->name);
         return 0;
     }
@@ -770,16 +760,16 @@ UInt8 srmConnectorSetCustomScanoutBuffer(SRMConnector *connector, SRMBuffer *buf
 
     if (srmConnectorModeGetWidth(mode) != srmBufferGetWidth(buffer) || srmConnectorModeGetHeight(mode) != srmBufferGetHeight(buffer))
     {
-        SRMError("[%s][%s] Failed to set custom scanout buffer. The buffer dimensions must match the connector's mode size.",
-                 connector->device->name,
+        SRMError("[%s] [%s] Failed to set custom scanout buffer. The buffer dimensions must match the connector's mode size.",
+                 connector->device->shortName,
                  connector->name);
         return 0;
     }
 
     if (srmBufferGetTextureID(connector->device, buffer) == 0)
     {
-        SRMError("[%s][%s] Failed to set custom scanout buffer. The buffer is not supported by the connector's device.",
-                 connector->device->name,
+        SRMError("[%s] [%s] Failed to set custom scanout buffer. The buffer is not supported by the connector's device.",
+                 connector->device->shortName,
                  connector->name);
         return 0;
     }
@@ -791,8 +781,8 @@ UInt8 srmConnectorSetCustomScanoutBuffer(SRMConnector *connector, SRMBuffer *buf
         if (!srmFormatIsInList(connector->currentPrimaryPlane->inFormats, buffer->scanout.fmt.format, buffer->scanout.fmt.modifier)
             || srmFormatIsInList(connector->currentPrimaryPlane->inFormatsBlacklist, buffer->scanout.fmt.format, buffer->scanout.fmt.modifier))
         {
-            SRMError("[%s][%s] Failed to set custom scanout buffer. Format not supported by the primary plane.",
-                     connector->device->name,
+            SRMError("[%s] [%s] Failed to set custom scanout buffer. Format not supported by the primary plane.",
+                     connector->device->shortName,
                      connector->name);
             return 0;
         }
@@ -833,8 +823,8 @@ UInt8 srmConnectorSetCustomScanoutBuffer(SRMConnector *connector, SRMBuffer *buf
 
     if (!bo)
     {
-        SRMError("[%s][%s] Failed to set custom scanout buffer. Could not get a GBM bo.",
-                 connector->device->name,
+        SRMError("[%s] [%s] Failed to set custom scanout buffer. Could not get a GBM bo.",
+                 connector->device->shortName,
                  connector->name);
         return 0;
     }
@@ -857,8 +847,8 @@ UInt8 srmConnectorSetCustomScanoutBuffer(SRMConnector *connector, SRMBuffer *buf
         UInt32 oldFmt = buffer->scanout.fmt.format;
         buffer->scanout.fmt.format = srmFormatGetAlphaSubstitute(buffer->scanout.fmt.format);
 
-        SRMError("[%s][%s] Failed to set custom scanout buffer. Format %s not supported by primary plane. Trying alpha substitute format %s",
-                 connector->device->name,
+        SRMError("[%s] [%s] Failed to set custom scanout buffer. Format %s not supported by primary plane. Trying alpha substitute format %s",
+                 connector->device->shortName,
                  connector->name,
                  drmGetFormatName(oldFmt),
                  drmGetFormatName(buffer->scanout.fmt.format));
@@ -866,8 +856,8 @@ UInt8 srmConnectorSetCustomScanoutBuffer(SRMConnector *connector, SRMBuffer *buf
         if (!srmFormatIsInList(connector->currentPrimaryPlane->inFormats, buffer->scanout.fmt.format, buffer->scanout.fmt.modifier)
             || srmFormatIsInList(connector->currentPrimaryPlane->inFormatsBlacklist, buffer->scanout.fmt.format, buffer->scanout.fmt.modifier))
         {
-            SRMError("[%s][%s] Failed to set custom scanout buffer. Unsupported format/modifier: %s - %s.",
-                     connector->device->name,
+            SRMError("[%s] [%s] Failed to set custom scanout buffer. Unsupported format/modifier: %s - %s.",
+                     connector->device->shortName,
                      connector->name,
                      drmGetFormatName(buffer->scanout.fmt.format),
                      drmGetFormatModifierName(buffer->scanout.fmt.modifier));
@@ -887,36 +877,36 @@ UInt8 srmConnectorSetCustomScanoutBuffer(SRMConnector *connector, SRMBuffer *buf
     {
         ret = drmModeAddFB2WithModifiers(
             connector->device->fd,
-            buffer->width, buffer->height,
+            buffer->dma.width, buffer->dma.height,
             buffer->scanout.fmt.format, handles, pitches, offsets, modifiers,
             &buffer->scanout.fb, DRM_MODE_FB_MODIFIERS);
     }
 
     if (ret)
     {
-        SRMError("[%s][%s] Failed to set custom scanout buffer using drmModeAddFB2WithModifiers(), trying drmModeAddFB2().",
-                 connector->device->name,
+        SRMError("[%s] [%s] Failed to set custom scanout buffer using drmModeAddFB2WithModifiers(), trying drmModeAddFB2().",
+                 connector->device->shortName,
                  connector->name);
 
         if (buffer->scanout.fmt.modifier != DRM_FORMAT_MOD_INVALID && buffer->scanout.fmt.modifier != DRM_FORMAT_MOD_LINEAR)
         {
-            SRMError("[%s][%s] Failed to set custom scanout buffer. drmModeAddFB2() and drmModeAddFB() do not support explicit modifiers.",
-                     connector->device->name,
+            SRMError("[%s] [%s] Failed to set custom scanout buffer. drmModeAddFB2() and drmModeAddFB() do not support explicit modifiers.",
+                     connector->device->shortName,
                      connector->name);
             goto releaseAndFail;
         }
 
         ret = drmModeAddFB2(
             connector->device->fd,
-            buffer->width, buffer->height,
+            buffer->dma.width, buffer->dma.height,
             buffer->scanout.fmt.format, handles, pitches, offsets,
             &buffer->scanout.fb, DRM_MODE_FB_MODIFIERS);
     }
 
     if (ret && planesCount == 1 && offsets[0] == 0)
     {
-        SRMError("[%s][%s] Failed to set custom scanout buffer using drmModeAddFB2(), trying drmModeAddFB().",
-                 connector->device->name,
+        SRMError("[%s] [%s] Failed to set custom scanout buffer using drmModeAddFB2(), trying drmModeAddFB().",
+                 connector->device->shortName,
                  connector->name);
 
         UInt32 depth, bpp;
@@ -924,8 +914,8 @@ UInt8 srmConnectorSetCustomScanoutBuffer(SRMConnector *connector, SRMBuffer *buf
 
         if (depth == 0 || bpp == 0)
         {
-            SRMError("[%s][%s] Failed to set custom scanout buffer using drmModeAddFB(), could not get depth and bpp for format %s.",
-                     connector->device->name,
+            SRMError("[%s] [%s] Failed to set custom scanout buffer using drmModeAddFB(), could not get depth and bpp for format %s.",
+                     connector->device->shortName,
                      connector->name,
                      drmGetFormatName(buffer->scanout.fmt.format));
             goto releaseAndFail;
@@ -933,15 +923,15 @@ UInt8 srmConnectorSetCustomScanoutBuffer(SRMConnector *connector, SRMBuffer *buf
 
         ret = drmModeAddFB(
             connector->device->fd,
-            buffer->width, buffer->height,
+            buffer->dma.width, buffer->dma.height,
             depth, bpp, pitches[0], handles[0],
             &buffer->scanout.fb);
     }
 
     if (ret)
     {
-        SRMError("[%s][%s] Failed to set custom scanout buffer using drmModeAddFB().",
-                 connector->device->name,
+        SRMError("[%s] [%s] Failed to set custom scanout buffer using drmModeAddFB().",
+                 connector->device->shortName,
                  connector->name);
         goto releaseAndFail;
     }
@@ -952,3 +942,19 @@ releaseAndFail:
     srmConnectorReleaseUserScanoutBuffer(connector, 0);
     return 0;
 }
+
+UInt32 srmConnectorGetFramebufferID(SRMConnector *connector)
+{
+    if (connector->state != SRM_CONNECTOR_STATE_INITIALIZED)
+        return 0;
+
+    return connector->renderInterface.getFramebufferID(connector);
+}
+
+EGLContext srmConnectorGetContext(SRMConnector *connector)
+{
+    if (connector->state != SRM_CONNECTOR_STATE_INITIALIZED)
+        return EGL_NO_CONTEXT;
+
+    return connector->renderInterface.getEGLContext(connector);
+}
