@@ -1,7 +1,7 @@
 #ifndef SRMCRTCPRIVATE_H
 #define SRMCRTCPRIVATE_H
 
-#include "../SRMCrtc.h"
+#include <SRMCrtc.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -24,6 +24,8 @@ struct SRMCrtcStruct
     SRMListItem *deviceLink;
     SRMConnector *currentConnector;
     struct SRMCrtcPropIDs propIDs;
+    UInt64 gammaSizeLegacy;
+    UInt64 gammaSize;
 };
 
 SRMCrtc *srmCrtcCreate(SRMDevice *device, UInt32 id);
